nsolid-console - the N|Solid Console Server
================================================================================

[![Build Status](http://badges.control-tower.nodesource.io/nsolid-console-ng/status.svg)](https://us-west-2.console.aws.amazon.com/codebuild/home?region=us-west-2#/projects/nsolid-console-ng-ci/view)

For more information on N|Solid, please see:

https://docs.nodesource.com/docs/



pre-reqs
================================================================================

N|Solid Console needs to be run with nsolid, so download and install it first.

[S3]: https://s3.console.aws.amazon.com/s3/buckets/nodesource-public-downloads/

Alternatively, it may be possible to run with vanilla node and a local install
of the `zmq` package. To get `zmq` installed, you will need a system install of
ØMQ, as noted below:

ØMQ - network messaging library
--------------------------------------------------------------------------------

### Mac

Install zeromq and libsodium via [brew][]:

[brew]: http://brew.sh/

    $ brew install libsodium

    ...
    🍺  /usr/local/Cellar/libsodium/1.0.13: 69 files, 1.2M

    $ brew install zeromq --with-libsodium

    ...
    ==> make
    ==> make install
    🍺  /usr/local/Cellar/zeromq/4.2.2: 76 files, 2.2M, built in 1 minute 28 seconds

If you run into an error like the following, it may be you have installed zmq,
but not with libsodium; you will need to `brew uninstall zmq` and reinstall it
(after installing libsodium):

    Error: zmq_curve_keypair operation failed. Method support in libzmq v4+ -with-libsodium.

As of the time of this writing, the zmq version is 4.2.2 .

### Ubuntu

You should already have build tools installed via apt-get.

    sudo apt-get install libtool pkg-config build-essential autoconf automake libzmq-dev

Install libsodium.

    git clone git://github.com/jedisct1/libsodium.git
    cd libsodium/
    ./autogen.sh
    ./configure
    make check
    sudo make install
    sudo ldconfig

Download [ØMQ Current Stable POSIX tarball](http://zeromq.org/intro:get-the-software)

    tar zxvpf zeromq-4.1.4.tar.gz
    cd zeromq-4.1.4/
    ./autogen.sh
    ./configure
    make check
    sudo make install
    sudo ldconfig



Running N|Solid Console Locally (after pre-reqs)
--------------------------------------------------------------------------------

Install dependencies

    npm install

Start the nsolid-console server with the following command:

    ./nsolid-console.js
    or
    ./nsolid-console.js | ./bole.js

To make the output more human friendly, pipe the output through the `bole`
logger, which redirects to stdout.

    ./nsolid-console.js | ./bole.js


The server runs in one of two modes:

* console server (default)
* relay server (`--relay`)


Console server mode
--------------------------------------------------------------------------------

When running in console mode, the default configuration will:

* create an API HTTP server listening on port 6753 on all network interfaces
* open zmq sockets on ports 9001, 9002, 9003 on all network interfaces
* store logs, data, etc in `~/.nsolid-console/`

The N|Solid Console and N|Solid CLI should be configured to point to the API
HTTP server url.

N|Solid processes should access the zmq sockets by setting the following
environment variables:

    NSOLID_COMMAND=localhost:9001
    NSOLID_DATA=localhost:9002
    NSOLID_BULK=localhost:9003


You can check the version of the API HTTP server by accessing the server's base
URL:

```
$ curl http://localhost:6753/api

{"versions":{"program":"3.0.0-pre","api":["3"]}}
```

Once you can confirm N|Solid Console is running start it up in your browser at:

    http://localhost:6753

There are two ways to have mock processes created:

[nsolid-mock](https://github.com/nodesource/nsolid-mock)

[nsolid-dev-demo](https://github.com/nodesource/nsolid-dev-demo)


You can either run `npm run watch` or `npm run watchify` for server reloading.

Other useful build scripts:

    "check-deps": "tools/check-deps.sh",
    "standard": "echo running standard && standard --verbose --parser babel-eslint",

relay mode
--------------------------------------------------------------------------------

To support proxying zmq through firewalls, buffering agent data during
console downtime, and scaling concerns, nsolid-console also supports running
in a "relay" mode. The relay sits between agents and a console server (or another relay). In this mode nsolid-console only looks like a console server
to agents and does not have any console server functionality.

To use the relay mode, both a `--relay` and a `--config` option must be
provided.

The config must contain three extra `[sockets]` settings pointing to an
upstream console server (or secondary relay):

```toml
[sockets]

commandRemoteAddr = "tcp://<console-address>:9001"
dataRemoteAddr = "tcp://<console-address>:9002"
bulkRemoteAddr = "tcp://<console-address>:9003"
```

See the default configuration file for information regarding additional
relay mode configuration settings.

--

When in relay mode, it it is possible to live-reload the
configuration via a `HUP` signal. Select configuration properties allow
this functionality, as listed below.

```toml
[sockets]
commandRemoteAddr
dataRemoteAddr
bulkRemoteAddr
publicKey

[relay]
maxBufferSizeMb
cleanupBatchSize
logSizeInterval
```

**Warning!**: If you are reloading to connect to a different Console server
with separate encryption keys, you **must** update the public key in the config
along with the sockets.
If you do not, **data will be lost as garbage** on the Console server end.

command line options
================================================================================

    -h --help            - print some help text, then exit
    -v --version         - print the program version, then exit
    -p --config-print    - print the configuration, then exit
    -c --config {file}   - use the specified TOML configuration file
    -r --relay           - run in Relay mode (requires `--config`)

The default config file is located in this directory tree at
`etc/default-config.toml` .  That config file contains additional comments
regarding values that can be set.

In addition, environment variables can be used to override the default
configuration values.  The environment variables names have a 1x1 correspondence
with the configuration properties in the config file.  Use the `-h` option to
list the names of the environment variables that can be used.



Tests
================================================================================

Unit tests
--------------------------------------------------------------------------------
Unit tests test individual, isolated pieces of the system. Most unit tests are
limited to a single module or library. The unit tests are located in `test`. You
can run these tests with the following command.

    npm t


Console tests
--------------------------------------------------------------------------------
Console tests test individual, isolated pieces of the console. You can run these
tests with the following command.

    npm run ctest


Other useful build scripts:

    "utest": "nsolid test/index.js | tap-spec",
    "itest": "nsolid test/integration/index.js | tap-spec",
    "ctest": "nsolid web/test/index.js | tap-spec",
    "integration": "npm -s run standard && npm -s run itest",
    "test": "npm -s run standard && npm -s run ctest && npm -s run utest",
    "tap-spec": "tap-spec",
    "all": "npm -s run standard && npm -s run ctest && npm -s run utest && npm -s run itest",



Dogfood
================================================================================

_N|Solid Console QA Environment_

Environments
--------------------------------------------------------------------------------
We have 4 environments for N|Solid Console Dogfood: `dev`, `staging`, `prod`,
and `relay`. Each environment can be deployed individually and on demand. The
`relay` environment is obviously the Console in relay mode. All of our agents
connect to this Console. The `relay` environment then sends the agent data to
the other environments. `dev` is used for testing feature branches, `staging` is
usually ran off of the master branch, and `prod` is generally only updated with
Release Candidates.

The N|Solid Release and Release Candidate pipelines will automatically update
the `prod` environment when they finish their builds. The `staging` environment
is updated every night with the `master` branch. The `dev` and `relay`
environments are only updated manually. Regardless, all of these can be
deployed manually in Slack.

_Please keep in mind, no matter what environment you are in, these are all
**production** processes. Any actions in any environment **will** affect that
process._

**URLs:**

http://relay.dogfood.nodesource.io

http://dev.dogfood.nodesource.io

http://staging.dogfood.nodesource.io

http://prod.dogfood.nodesource.io

_You must be on the VPN to access them_


Commands
--------------------------------------------------------------------------------
Dogfood is managed by [Control Tower](https://github.com/nodesource/control-tower).
These commands must all be executed in Slack in any channel. When a deployment
has completed with either success or failure, Control Tower will post a message
in `#github-nsolid`.

**Deploy**

_Basic Syntax:_

`/ct deploy console <env> <branch> [--with-node]`

Starts a deployment from a given branch to a given environment.

- Environment can be one of: `dev`, `staging`, `prod`, or `relay`

- Branch can be any branch in the `nsolid-console-ng` repo. If no branch is
supplied, it will default to `master`.

- The `--with-node` flag can be used if you need to build `nsolid-node` off of a
branch instead of using the latest release version.

**Status**

`/ct deploy status console`

Returns the date and time of the last deploy to all environments, as well what
branch is currently deployed.

`/ct pipeline status dogfood-console`

Check the status of the Codepipeline execution that is building your branch.

`/ct pipeline status dogfood-console-with-node`

Check the status of the Codepipeline execution that is building your branch if
you supplied the `--with-node` flag.

**Examples:**

Deploy branch `username/feature` to `dev` environment

    /ct deploy console dev username/feature

Deploy branch `username/feature` to `dev` environment with an nsolid-node
branch of `username/node`

    /ct deploy console dev username/feature --with-node=username/node

Deploy `master` branch to `staging` environment

    /ct deploy console staging
