'use strict'

exports.handleRequest = handleRequest

const path = require('path')

const util = require('../../util')
const Logger = util.getLogger(__dirname, __filename)

const fileSender = require('../file-sender')

// Handle the HTTP request
function handleRequest (req, res) {
  const context = req.nsConsole.context
  const config = context.config
  const qData = req.nsConsole.qData
  const diskAssetManager = context.diskAssetManager

  const assetPath = path.join(config.data.dir, 'assets')
  const asset = diskAssetManager.byId(qData.id)

  if (asset == null) {
    res.status(404).send('asset not found')
    return
  }

  let fileName = path.join(assetPath, asset.fileName)
  const fileZipped = asset.compressed
  const headers = {
    'Content-Type': 'application/octet-stream',
    'Content-Disposition': 'attachment; filename=' + path.basename(fileName)
  }

  if (!fileZipped) {
    fileSender.sendFile(req, res, fileName, fileZipped, headers)
    return
  }

  // file is compressed, so reference the compressed file name
  fileName = `${fileName}.gz`

  Logger.debug(`sending compressed asset ${qData.id}: ${fileName}`)
  fileSender.sendFile(req, res, fileName, fileZipped, headers)
}
