'use strict'

exports.handleRequest = handleRequest

// seconds
const DEFAULT_INTERVAL = 1

const httpUtils = require('../http-utils')
const normalizers = require('./normalizers')

const util = require('../../util')
const Logger = util.getLogger(__dirname, __filename)

// Handle the HTTP request
function handleRequest (req, res) {
  const context = req.nsConsole.context
  const eventStream = req.nsConsole.eventStream
  const qData = req.nsConsole.qData
  const interval = qData.interval || DEFAULT_INTERVAL
  const queryExpr = qData.q
  const fields = qData.field.filter(field => field.domain === 'metrics')

  Logger.debug('handling metrics request')

  poll()
  httpUtils.syncPollTillDone(eventStream, interval * 1000, null, poll)

  function poll () {
    const agents = context.agentManager.list()

    for (let agent of agents) {
      if (agent.metrics == null || agent.info == null) continue
      if (queryExpr != null && !queryExpr.evalForAgent(agent)) continue

      const header = {
        time: Date.now(),
        id: agent.id,
        app: agent.info.app,
        hostname: agent.info.hostname,
        tags: agent.info.tags
      }

      let agentMetrics = agent.metrics
      if (fields.length !== 0) {
        agentMetrics = {}
        for (let field of fields) {
          agentMetrics[field.name] = field.valueFromAgent(agent)
        }
      }

      const metrics = normalizers.agentInfo(Object.assign(header, agentMetrics))

      eventStream.writeData(metrics)
    }
  }
}
