'use strict'

exports.handleRequest = handleRequest

const util = require('../../util')
const Logger = util.getLogger(__dirname, __filename)

// Handle the HTTP request
function handleRequest (req, res) {
  const context = req.nsConsole.context
  const qData = req.nsConsole.qData

  const commandParams = {
    command: qData.name,
    args: qData.data,
    id: qData.id
  }

  const requestId = context.commandDispatcher.send(commandParams)

  const revokeEnd = context.commandDispatcher.revocableOnce(`end:${requestId}`, () => {
    Logger.warn('Unexpected end of command', commandParams)
    res.status(500).send({ error: { message: 'Unexpected end of command.' } })
    endCommand(context, requestId)
  })

  const revokeError = context.commandDispatcher.revocableOnce(`error:${requestId}`, err => {
    Logger.error('Error running command', err, commandParams)
    res.status(500).send({ error: { message: err.message } })
    endCommand(context, requestId)
  })

  const revokeData = context.commandDispatcher.revocableOnce(`data:${requestId}`, data => {
    const agent = context.agentManager.byId(data.agentId)
    const response = {
      result: data.body,
      id: agent.id,
      app: agent.tags.app,
      hostname: agent.tags.hostname,
      tags: (agent.info && agent.info.tags) ? agent.info.tags : [],
      time: data.time
    }
    res.send(response)
    endCommand(context, requestId)
  })

  function endCommand (requestId) {
    revokeEnd()
    revokeError()
    revokeData()
  }
}
