'use strict'

const util = require('../../util')
const Logger = util.getLogger(__dirname, __filename)

module.exports = function (context) {
  const name = 'exit'
  function run (messageObj) {
    let agent = context.agentManager.byId(messageObj.agentId)

    if (agent == null) return

    Logger.debug(`Removing exited agent ${agent.id}`)
    context.agentManager.markDead(agent)

    const exitEvent = {}

    if (agent.info) {
      if (agent.info.app) exitEvent.app = agent.info.app
      if (agent.info.hostname) exitEvent.hostname = agent.info.hostname
      if (agent.info.tags) exitEvent.tags = agent.info.tags
      if (agent.info.pid) exitEvent.pid = agent.info.pid
    }

    if (messageObj.exit_code != null) exitEvent.exitCode = messageObj.exit_code
    if (messageObj.error != null) exitEvent.error = messageObj.error
    if (messageObj.profile_id) exitEvent.profileId = messageObj.profile_id

    context.eventManager.emitSystemEvent('agent-exit', agent.id, exitEvent)
  }

  function predicate (messageObj) {
    return messageObj.command === name
  }

  return { name, run, predicate }
}
