#!/bin/bash

ARCH=$(uname)
NVM=$(type ~/.nvm/nvm.sh 2> /dev/null)
DOCKER=$(which docker)

# Check if OSX
if [[ $ARCH != 'Darwin' ]]; then
  echo "This script can only be run on Darwin. Please try again"
  exit 1
fi

# Check if NVM installed
if [[ -z $NVM ]]; then
  echo "Node Version Manager is needed to run this script. Please install it with:"
  echo "curl -o- https://raw.githubusercontent.com/creationix/nvm/v0.33.6/install.sh | bash"
  exit 1
fi
. ~/.nvm/nvm.sh
. ~/.bashrc

# Check if Docker installed
if [[ -z $DOCKER ]]; then
  echo "Docker is needed to run this script. Get it here:"
  echo "https://www.docker.com/community-edition#/download"
  exit 1
fi

echo ""
echo "---"
echo "Starting prebuild updates..."
echo "---"
echo ""

echo "Logging into AWS ECR..."
`aws ecr get-login --no-include-email`

SCRIPTDIR=$(
  cd $(dirname "$0")
  pwd
)

if [[ -d $SCRIPTDIR/output ]]; then
  rm -rf $SCRIPTDIR/output
fi

mkdir -p $SCRIPTDIR/output
cd $SCRIPTDIR/output
rsync -rvq --exclude=output --exclude=lib ../../* ./
mkdir -p lib

echo ""
echo "---"
echo "Starting Darwin..."
echo "---"
echo ""

nvm install 6
node install.js

nvm install 8
node install.js

nvm install 10
node install.js

rm -rf ../../lib/darwin*
mv lib/darwin* ../../lib/

echo ""
echo "---"
echo "Darwin Complete."
echo "---"
echo ""
echo "---"
echo "Starting Linux x64..."
echo "---"
echo ""

docker run -v ~/.ssh/:/root/.ssh -v $(pwd):/root/leveldown-build --entrypoint /root/leveldown-build/tools/prebuild-scripts/update-x64.sh 800406105498.dkr.ecr.us-west-2.amazonaws.com/codebuild:build-binary-linux

rm -rf ../../lib/linux-x64*
mv lib/linux-x64* ../../lib/

echo ""
echo "---"
echo "Linux x64 Complete."
echo "---"
echo ""
echo "---"
echo "Starting Linux ARM..."
echo "---"
echo ""

docker run -v ~/.ssh/:/root/.ssh -v $(pwd):/root/leveldown-build --entrypoint /root/leveldown-build/tools/prebuild-scripts/update-arm.sh 800406105498.dkr.ecr.us-west-2.amazonaws.com/codebuild:build-binary-arm

rm -rf ../../lib/linux-arm*
mv lib/linux-arm* ../../lib/

echo ""
echo "---"
echo "Linux ARM Complete."
echo "---"
echo ""
echo "---"
echo "Starting Alpine Linux [NodeJS Boron]..."
echo "---"
echo ""

docker run -v ~/.ssh/:/root/.ssh -v $(pwd):/root/levedown-build --entrypoint /root/leveldown-build/tools/prebuild-scripts/update-alpine.sh 800406105498.dkr.ecr.us-west-2.amazonaws.com/codebuild:build-binary-alpine

rm -rf ../../lib/alpine-x64-48
mv lib/alpine* ../../lib/

echo ""
echo "---"
echo "Alpine Linux [NodeJS Boron] Complete."
echo "---"
echo ""
echo "---"
echo "Starting Alpine Linux [NodeJS Carbon]..."
echo "---"
echo ""

docker run -v ~/.ssh/:/root/.ssh -v $(pwd):/root/leveldown-build --entrypoint /root/leveldown-build/tools/prebuild-scripts/update-alpine.sh 800406105498.dkr.ecr.us-west-2.amazonaws.com/codebuild:build-binary-alpine-carbon

rm -rf ../../lib/alpine-x64-57
mv lib/alpine* ../../lib/

echo ""
echo "---"
echo "Alpine Linux [NodeJS Carbon] Complete."
echo "---"
echo ""

rm -rf $SCRIPTDIR/output

docker run -v ~/.ssh/:/root/.ssh -v $(pwd):/root/leveldown-build --entrypoint /root/leveldown-build/tools/prebuild-scripts/update-alpine.sh 800406105498.dkr.ecr.us-west-2.amazonaws.com/codebuild:build-binary-alpine-dubnium

rm -rf ../../lib/alpine-x64-57
mv lib/alpine* ../../lib/

echo ""
echo "---"
echo "Alpine Linux [NodeJS Dubnium] Complete."
echo "---"
echo ""

rm -rf $SCRIPTDIR/output

echo ""
echo "---"
echo "All prebuilt native modules have been updated."
echo "---"
echo ""
exit 0
