#!/usr/bin/env node

'use strict'

const pj = require('./package.json')
const version = pj.version
const platform = require('./platform')()
const fs = require('fs')
const path = require('path')
const rimraf = require('rimraf')
const npm = require(path.dirname(process.argv[0]) + '/../lib/node_modules/npm/lib/npm.js')

console.log(`Checking for Snapdiff ${version} for ${platform}`)

const platDir = `./lib/${platform}`

fs.stat(`${platDir}/snapdiff`, err => {
  if (!err) {
    console.log(`Already installed.  Remove ${platDir}/snapdiff if you want to reinstall it.`)
    return
  }

  npm.load({ save: false, prefix: './lib' }, err => {
    if (err) throw err
    npm.install('git+ssh://git@github.com/nodesource/snapdiff.git#v0.3.2', err => {
      if (err) throw err
      try {
        fs.statSync(platDir)
      } catch (err) {
        fs.mkdirSync(platDir)
      }
      const deps = require('./lib/node_modules/snapdiff/package.json').dependencies
      pj.dependencies = deps
      fs.writeFileSync('package.json', JSON.stringify(pj, null, '  ') + '\n')
      fs.rename('./lib/node_modules/snapdiff', `${platDir}/snapdiff`, err => {
        if (err) throw err
        rimraf(`${platDir}/snapdiff/src`, err => {
          if (err) throw err
        })
        rimraf(`${platDir}/snapdiff/deps`, err => {
          if (err) throw err
        })
        rimraf(`${platDir}/snapdiff/test`, err => {
          if (err) throw err
        })
        rimraf(`${platDir}/snapdiff/assets`, err => {
          if (err) throw err
        })
        rimraf(`${platDir}/snapdiff/tools`, err => {
          if (err) throw err
        })
        rimraf(`${platDir}/snapdiff/build/Release/.deps`, err => {
          if (err) throw err
        })
        rimraf(`${platDir}/snapdiff/build/Release/obj.target`, err => {
          if (err) throw err
        })
      })
    })
  })
})
