## Overhead of Obtaining Snapshot Aggregates

The leak we manufactured for these measurements:

```js
function serveIndex(res) {
  for (var i = 0; i < leaksPerRequest; i++) {
    leak.push(`${i} of ${leaksPerRequest} at ${process.hrtime()[1]}ns`)
  }
  res.writeHead(200)
  res.end()
}
```

The `process.hrtime` part was included to ensure that we create unique strings (v8 is smart).
As a matter of fact v8 seems to never store that string as one, but as 3 parts. 2 of those will be repeated so this way
memory is saved (I'm assuming it is stored that way, at least in the snapshot it shows up as separate strings).

```js
[ '1 of'
, '100000'
, 'at 32423'
]
```

### Taking Snapshots

Snapshot were obtained via the heapdump module at different stages of the leak.

| Process RSS  | Snapshot Size   |
| -------------|---------------- |
|   27M        |     3M          |
|   81M        |    41M          |
|  395M        |    79M          |
|  577M        |   117M          |
|  820M        |   156M          |
|  878M        |   194M          |
|  961M        |   233M          |
| 1.02G        |   271M          |
| 1.12G        |   311M          |
| 1.16G        |   340M          |
| 1.18G        |   388M          |
| 1.25G        |   508M          |
| 1.50G        |   546M          |
| 1.52G        |   628M          |

![assets/snapshot-by-process-rss](../assets/snapshot-by-process-rss.png)

As you can see, the size of the snapshot doesn't reflect 1:1 what's on the heap and/or the RSS.
However one important observation is that snapshot size is about 1/3 of the RSS of the process it was taken for.

The latter is important to consider when parsing the table below. In short from the _very limited_ findings here we can
conclude that it is very unlikely to encounter a snapshot larger than 300MB.

### Obtaining Aggregates Command

```
> \time -pl ./bin/main --agg "_samples/leaking-strings/$SNAPSHOT" > res.json && \
du -h "_samples/leaking-strings/$SNAPSHOT" && ls -l res.json
```

### Result

| Snapshot Size  | Time    | Max RSS Used  | Aggregate JSON Size   |
| ---------------|-------- |-------------- | ----------------------|
|      3M        |   0.24s |    12M        | 6.8K                  |
|     41M        |   3.38s |   158M        | 7.2K                  |
|     79M        |   6.63s |   298M        | 7.0K                  |
|    117M        |   9.68s |   460M        | 7.1K                  |
|    156M        |  12.93s |   585M        | 7.2K                  |
|    194M        |  15.99s |   781M        | 7.2K                  |
|    233M        |  19.35s |   907M        | 7.2K                  |
|    271M        |  22.63s |  1.02G        | 7.0K                  |
|    311M        |  26.05s |  1.14G        | 7.2K                  |
|    340M        |  29.78s |  1.27G        | 7.2K                  |
|    388M        |  32.81s |  1.41G        | 7.2K                  |
|    508M        |  41.91s |  1.88G        | 7.0K                  |
|    546M        |  45.00s |  1.98G        | 7.2K                  |
|    628M        |  51.12s |  2.24G        | 7.2K                  |
|                |         |               |                       |
|                |         |               |                       |
|    2.1G        | 182.55s |  6.07G        | 4.1K                  |

The last snapshot was unrelated to the previous ones but serves as an example of [a very large snapshot that crashes
DevTools](https://github.com/nodesource/nsolid-console/issues/285) but _snapdiff_ parses and processes just fine.

![assets/processing-time](../assets/processing-time.png)

![assets/memory-overhead](../assets/memory-overhead.png)

All data shown in tables and images is availabe as a [google
sheet](https://docs.google.com/spreadsheets/d/137fyO_AmeXDv4jbzv2SHsSennGjhtIybKyrS2k1o9Qg/edit?usp=sharing).

Time and Memory Overhead are almost linear to snapshot size. We didn't graph the aggregate json size since that doesn't
change much as only the numbers change but not more rows are added.
